--[[
                                                    ----o----(||)----oo----(||)----o----

                                                            v1.01 - 26th May 2021
                                                    Copyright (C) Taraezor / Chris Birch

                                                    ----o----(||)----oo----(||)----o----
]]


local _, NetherwingEggs = ...
NetherwingEggs.points = {}
NetherwingEggs.textures = {}
NetherwingEggs.scaling = {}

local db
local defaults = { profile = { icon_scale = 1.4, icon_alpha = 0.8, icon_choice = 1, showCoords = true } }

local continents = {}

-- upvalues
local GameTooltip = _G.GameTooltip
local GetSubZoneText = _G.GetSubZoneText
local IsControlKeyDown = _G.IsControlKeyDown
local IsIndoors = _G.IsIndoors
local LibStub = _G.LibStub
local UIParent = _G.UIParent
local format = _G.format
local next = _G.next

local HandyNotes = _G.HandyNotes
local TomTom = _G.TomTom

-- The original Netherwing Eggs purple theme
local pc_colour_Prefix		= "\124cFF8258FA"
local pc_colour_Highlight	= "\124cFFB19EFF"
local pc_colour_PlainText	= "\124cFF819FF7"

-- I use this for debugging
local function printPC( message )
	if message then
		DEFAULT_CHAT_FRAME:AddMessage( pc_colour_Prefix.. "Netherwing Eggs".. ": ".. pc_colour_PlainText.. message.. "\124r" )
	end
end

-- plugin handler for HandyNotes
local function infoFromCoord(mapFile, coord)
	local point = NetherwingEggs.points[mapFile] and NetherwingEggs.points[mapFile][coord]
	return point[1], point[2], point[3]
end

function NetherwingEggs:OnEnter(mapFile, coord)
	if self:GetCenter() > UIParent:GetCenter() then -- compare X coordinate
		GameTooltip:SetOwner(self, "ANCHOR_LEFT")
	else
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	end

	local mvq, note, author = infoFromCoord(mapFile, coord)

	if mvq == "Q" then
		GameTooltip:SetText( pc_colour_Prefix .."The Not-So-Friendly Skies..." )
	else
		GameTooltip:SetText( pc_colour_Prefix .."Netherwing Egg" )
	end
	
	if note then
		GameTooltip:AddLine( note )
	end
	if db.showCoords == true then
		local mX, mY = HandyNotes:getXY(coord)
		mX, mY = mX*100, mY*100
		GameTooltip:AddLine( "(" ..format( "%.02f", mX ) .."," ..format( "%.02f", mY ) ..")" )
	end

	if TomTom then
		GameTooltip:AddLine("Right-click to set a waypoint", 1, 1, 1)
		GameTooltip:AddLine("Control-Right-click to set waypoints to every |cFF0070DENetherwing Egg|r", 1, 1, 1)
	end

	GameTooltip:Show()
end

function NetherwingEggs:OnLeave()
		GameTooltip:Hide()
end

local function createWaypoint(mapID, coord)
	local x, y = HandyNotes:getXY(coord)
	TomTom:AddWaypoint(mapID, x, y, { title = "Netherwing Egg", persistent = nil, minimap = true, world = true })
end

local function createAllWaypoints()
	for mapFile, coords in next, NetherwingEggs.points do
		if not continents[mapFile] then
			for coord in next, coords do
				if coord then
					createWaypoint(mapFile, coord)
				end
			end
		end
	end
	TomTom:SetClosestWaypoint()
end

function NetherwingEggs:OnClick(button, down, mapFile, coord)
	if TomTom and button == "RightButton" and not down then
		if IsControlKeyDown() then
			createAllWaypoints()
		else
			createWaypoint(mapFile, coord)
		end
	end
end

do
	local _, _, _, version = GetBuildInfo()
	NetherwingEggs.continents = (version < 30000) and 1945 or 101
	NetherwingEggs.valley = (version < 30000) and 1948 or 104
	continents[NetherwingEggs.continents] = true

    local bucket = CreateFrame("Frame")
    bucket.elapsed = 0
    bucket:SetScript("OnUpdate", function(self, elapsed)
        self.elapsed = self.elapsed + elapsed
        if self.elapsed > 1.5 then
            self.elapsed = 0
			local insideMine = ( GetSubZoneText() == "Netherwing Mines" and IsIndoors() ) and true or false
			if NetherwingEggs.insideMine == nil then
				NetherwingEggs.insideMine = insideMine
			elseif NetherwingEggs.insideMine ~= insideMine then
				NetherwingEggs.insideMine = insideMine
				NetherwingEggs:Refresh()
			end
        end
    end)

	if NetherwingEggs.insideMine == nil then
		NetherwingEggs.insideMine = ( GetSubZoneText() == "Netherwing Mines" and IsIndoors() ) and true or false
	end

	-- custom iterator we use to iterate over every node in a given zone	
			
	local function iterator(t, prev)
		if not t then return end
		local coord, v = next(t, prev)
		while coord do
			if v then
				if v[1] == "M" then
					if NetherwingEggs.insideMine == true then
						return coord, nil, NetherwingEggs.textures[db.icon_choice], db.icon_scale * NetherwingEggs.scaling[db.icon_choice], db.icon_alpha
					end
				else
					if NetherwingEggs.insideMine == false then
						return coord, nil, NetherwingEggs.textures[db.icon_choice], db.icon_scale * NetherwingEggs.scaling[db.icon_choice], db.icon_alpha
					end
				end
			end
			coord, v = next(t, coord)
		end
	end
	function NetherwingEggs:GetNodes2(mapID)
		return iterator, NetherwingEggs.points[mapID]
	end
end

-- Interface -> Addons -> Handy Notes -> Plugins -> Netherwing Eggs options
local options = {
	type = "group",
	-- Localise the name and description later
	name = "Netherwing Eggs",
	desc = "Location of Netherwing Eggs",
	get = function(info) return db[info[#info]] end,
	set = function(info, v)
		db[info[#info]] = v
		NetherwingEggs:Refresh()
	end,
	args = {
		desc = {
			name = "These settings control the look and feel of the " ..pc_colour_Prefix .."Netherwing Eggs" .. "\124r icon",
			type = "description",
			order = 1,
		},
		icon_scale = {
			type = "range",
			name = "Icon Scale",
			desc = "Change the size of the icons",
			min = 0.25, max = 2, step = 0.01,
			arg = "icon_scale",
			order = 2,
		},
		icon_alpha = {
			type = "range",
			name = "Icon Alpha",
			desc = "Change the transparency of the icons",
			min = 0, max = 1, step = 0.01,
			arg = "icon_alpha",
			order = 3,
		},
		icon_choice = {
			type = "range",
			name = "Icon Choice",
			desc = "1 = Phasing\n2 = Raptor Egg\n3 = Stars\n4 = Purple\n5 = White\n6 = Mana Orb\n"..
					"7 = Cogwheel\n8 = Frost\n9 = Diamond\n10 = Red\n11 = Yellow\n12 = Green\n13 = Screw",
			min = 1, max = 13, step = 1,
			arg = "icon_choice",
			order = 4,
		},
		showCoords = {
			name = "Show coordinates",
			desc = "(xx.xx,yy.yy)",
			type = "toggle",
			width = "full",
			arg = "showCoords",
			order = 5,
		},
	},
}

function NetherwingEggs:OnEnable()
	local HereBeDragons = LibStub("HereBeDragons-2.0", true)
	if not HereBeDragons then
		HandyNotes:Print("Your installed copy of HandyNotes is out of date and the " ..pc_colour_Prefix .."Netherwing Eggs" .. 
						"\124r plug-in will not work correctly.  Please update HandyNotes to version 1.5.0 or newer.")
		return
	end
	for continentMapID in next, continents do
		local children = C_Map.GetMapChildrenInfo(continentMapID, nil, true)
		for _, map in next, children do
			local coords = NetherwingEggs.points[map.mapID]
			if coords then
				for coord, criteria in next, coords do
					local mx, my = HandyNotes:getXY(coord)
					local cx, cy = HereBeDragons:TranslateZoneCoordinates(mx, my, map.mapID, continentMapID)
					if cx and cy then
						NetherwingEggs.points[continentMapID] = NetherwingEggs.points[continentMapID] or {}
						NetherwingEggs.points[continentMapID][HandyNotes:getCoord(cx, cy)] = criteria
					end
				end
			end
		end
	end
	HandyNotes:RegisterPluginDB("NetherwingEggs", self, options)
	db = LibStub("AceDB-3.0"):New("HandyNotes_NetherwingEggsDB", defaults, "Default").profile
	NetherwingEggs:Refresh()
	NetherwingEggs.Enabled = true
end

function NetherwingEggs:Refresh()
	self:SendMessage("HandyNotes_NotifyUpdate", "NetherwingEggs")
end

LibStub("AceAddon-3.0"):NewAddon(NetherwingEggs, "HandyNotes_NetherwingEggsDB", "AceEvent-3.0")